/*
jQuery simpleLightbox
par Gildas P. / www.gildasp.fr
en 03/2022

largement inspiré de lightbox_me https://buckwilson.me/lightboxme/
mais plus moderne (utilsie flex au lieu de calucls js un peu foireux), 
avec des  fonctionnalités supplémentaires.

- clone le sujet s'il est visible, mais pas s'il est caché
- adjustContent : true - ajuste la taille du contenu (image) en option 
  (pour un contenu complexe vaut mieux le faire manuellement)
- navSiblings : true - possibilité de faire un diaporama des siblings :)
	avec des boutons .next et .prev auto-générés
	et même de customiser l'animation de transition (mais c'est chaud)
- possibilité de styler en css l'overlay via overlayManualStyle:true (plutôt qu'en js)
	l'overlay c'est .lightbox_overlay (modifiable via le param overlayClass)
*/

(function($){
	$.fn.simpleLightbox = function(params){

		$(this).each(function(){

			var opts = $.extend({

		        // close button
		        closeSelector: ".close", // tous les élements .close du contenu fermeront la lightbox au clic
		        closeClick: true, // fermer au clic sur .close
		        closeEsc: true, // fermer avec la touche ESC

		        // callbacks
		        onOpen: function($this) {},
		        onClose: function($this) {},

	    		zIndex: 999,

	    		// contents
	    		contentSpeed: 300,
	    		cloneContent: true, // pour laisser le contenu à sa place, et afficher un clone en lightbox
	    		adjustContent: false, // empêche le contenu de dépasser de l'écran
	    		contentWidth: '80%',
	    		contentHeight: '80%',
	    		contentFit: 'contain',
	    		centered: true, // contenu centré avec flex, en js
	    		contentPosition: 'fixed', // ou 'absolute' pour permettre le scroll
	    		contentClass: 'lightbox_content', // ajoute cette classe au contenu, pour pouvoir ajouter du style en css
	    		

	    		// gallery mode
	    		navSiblings: false, // pour activer les boutons précédent/suivant
	    		nextSiblingOnClick: false, // pour passer au suivant au clic sur le contenu
	    		navSiblingsClass: 'lightbox_nav', // + .prev et .next
	    		siblingInSpeed: 350,
	    		siblingOutSpeed: 250,
	    		nextContent: '→',
	    		prevContent: '←',
	    		siblingOutFunc: function(way, $obj, callback){ // way -1 ou 1
	    			return $obj.fadeOut(opts.siblingOutSpeed, callback); // important d'appeler callback dans l'anim 
	    		},
	    		siblingInFunc: function(way, $obj){
	    			return $obj.fadeIn(opts.siblingInSpeed);
	    		},

	    		// overlay
	    		showOverlay: true,
	    		overlayClass: 'lightbox_overlay',
	    		overlaySpeed: 250,
	    		overlayManualStyle: true, // styler l'overlay manuellement via la feuille de style
	    		// overlayStyle: { background: 'black', opacity: .6 }, // sinon, style par défaut en js
	    		overlayStyle: '',
			}, params);

			$this = $(this);

			// ajout de l'overlay
			if (opts.showOverlay) {
	            //check if there's an existing overlay, if so, make subequent ones clear
	            var $currentOverlays = $("."+opts.overlayClass);
	            if ($currentOverlays.length == 0){
	                $overlay = $('<div class="'+opts.overlayClass+'"></div>');
	                $('body').append($overlay);
	                $overlay.css({ 
	                	position: 'fixed', width: '100vw', height: '100vh', 
	                	top: 0, left: 0, zIndex: (opts.zIndex + 2), 
	                	display: 'none' 
	                });
	            } else {
	            	$overlay = $('.'+opts.overlayClass);
	            }
	        }

	        // style de l'overlay
	        /*if(opts.showOverlay && !opts.overlayManualStyle){
	        	$overlay.css(opts.overlayStyle);
	        }*/
	        // nan, style via le css par défaut
	        if(opts.overlayStyle != ''){
	        	$overlay.css(opts.overlayStyle);
	        }

	        // ouverture de l'overlay
	        if (opts.showOverlay) {
	            $overlay.fadeIn(opts.overlaySpeed, function() {
	            	setContent();
	                opts.onOpen($this);
	            });
	        } else {
	            setContent();
	            opts.onOpen($this);
	        }

	        // events
	        if(opts.closeEsc){ 
	        	$(window).on('keyup.lightbox', observeKeyPress);
	        }
	        /*if (opts.closeClick) {
	            $overlay.click(function(e) { closeLightbox(); e.preventDefault; });
	        }*/

	        /*$this.delegate(opts.closeSelector, "click", function(e) {
	            closeLightbox(); e.preventDefault();
	        });*/
	        $this.on('close', closeLightbox); // pour trigger manuellement la fermeture...

	        // private functions
	        function closeLightbox() {

	        	if($box == null){ // rustine pour un cas part. étrange
	        		$box = $('.lightbox_box');
	        	}

	            if (opts.cloneContent) {
	            	if(opts.showOverlay){
	            		$box.add($overlay).remove();
	            	} else {
	            		$box.remove();
	            	}
	                
	            } else {
	            	if(opts.showOverlay){
	                	$box.add($overlay).hide();
	                } else {
						$box.hide();
	                }
	            }

	            opts.onClose($this);

				// clean up events
	            $this.off('close', closeLightbox);
	            $this.off('click.lightbox_sibling');
	            
	            $(window).off('keyup.lightbox');

	            var $closer = $box.find(opts.closeSelector);
		        if($closer.length>0){
		        	$closer.off("click.lightbox");
		        } else if(opts.closeClick){
		        	$box.off("click.lightbox");
		        }
	        }

	        function observeKeyPress(e) {
	            if((e.keyCode == 27 || (e.DOM_VK_ESCAPE == 27 && e.which==0)) && opts.closeEsc) closeLightbox();
	        }

	        var $box = null;

	        function cloneOrNot($this, insertAfter, $prev){
	        	if(typeof insertAfter == 'undefined'){ insertAfter = false; }
	        	if($this.css('display')!='none' || opts.cloneContent == true){
	        		// contenu visible ! -> on clone
	        		opts.cloneContent = true;
	        	}
	        	if(opts.cloneContent){
	        		$this = $this.clone();
	        		/*if($this.css('display')=='none'){
	        			$this.css('display', 'block');
	        		}*/
	        	}

	        	// on borde l'apparition du contenu (éventuellement caché)
	        	// $this.show();
	        	setTimeout(function(){ $this.show(); }, 50);

        		if(insertAfter){
        			$prev.after($this.addClass(opts.contentClass).show());
        		} else {
        			$box.append($this.addClass(opts.contentClass).show());
        		}
	        	return $this;
	        		
	        	/*} else {
	        		if(insertAfter){
	        			$prev.after($this.addClass(opts.contentClass).show());
	        		} else {
	        			$box.append($this.addClass(opts.contentClass).show());
	        		}
	        	}*/
	        }
	        function adjustOrNot($this){
	        	if(opts.adjustContent){
	        		// var $child = $box.children().first();
	        		var $child = $box.find('.'+opts.contentClass).first();
	        		if(!$child.is('img') && !$child.is('video')){
	        			$child.css({
		        			width: opts.contentWidth,
		        			height: opts.contentHeight
		        		});
	        			$child = $child.find('img, video').first();
		        		$child.css({
		        			width: '100%',
		        			height: '100%',
		        			'object-fit': opts.contentFit
		        		});
	        		} else {
		        		$child.css({
		        			width: opts.contentWidth,
		        			height: opts.contentHeight,
		        			'object-fit': opts.contentFit
		        		});
		        	}
	        	}
	        }
	        function setContent(){
	        	// créer un conteneur
	        	$box = $('<div class="lightbox_box"></div>');
	        	// $box.hide();
	        	$('body').append($box);

	        	// conteneur flex ?
	        	if(opts.centered){
	        		$box.css({
	        			position: opts.contentPosition, width:'100vw', height:'100vh', top:0, left:0, 'z-index':opts.zIndex+3,
	        			display: 'flex', 'flex-wrap': 'wrap', 'justify-content': 'center', 'align-items': 'center' });
	        	}
	        	$box.hide();

	        	// clone ?
	        	$tmp = cloneOrNot($this);

	        	// adjust content size ?
	        	adjustOrNot($this);

	        	// events        	
		        var $closer = $box.find(opts.closeSelector);
		        if($closer.length>0){
		        	$closer.on("click.lightbox", function(e) {
			            closeLightbox(); e.preventDefault();
			        });
		        } else if(opts.closeClick){
		        	$box.on("click.lightbox", function(e) {
			            closeLightbox(); e.preventDefault();
			        });
		        }

		        // nav through siblings ?
		        if(opts.navSiblings){ // ---------------------------------

		        	// boutons de navigation
		        	$prev = $('<div class="'+opts.navSiblingsClass+' prev">'+opts.prevContent+'</div>');
		        	$next = $('<div class="'+opts.navSiblingsClass+' next">'+opts.nextContent+'</div>');
		        	$box.prepend($prev);
		        	$box.append($next);

		        	$prev.on('click', function(e){ 
		        		e.preventDefault();
		        		e.stopPropagation();
		        		goBy(-1);
		        	});
		        	$next.on('click', function(e){ 
		        		e.preventDefault();
		        		e.stopPropagation();
		        		goBy(1);
		        	});

		        	// trouver les siblings
		        	var $siblings = $this.parent().children();
		        	// console.log("siblings", $siblings.length);

		        	// rang du sibling actuel...
		        	var index = 0;
		        	$siblings.each(function(i){
		        		if($(this).is($this)){
		        			index = i;
		        		}
		        	});
		        	// console.log("current index is", index);

		        	function goBy(num){
		        		goTo(index+num, num);
		        	}
		        	function goTo(num, way){
		        		// console.log("goTo", num);
		        		if(num < $siblings.length && num>=0){

			        		// disparition du contenu actuel + remove it
			        		// $box.find('.'+opts.contentClass).stop()
			        		// .fadeOut(opts.siblingOutSpeed, function(){ // out func

			        		opts.siblingOutFunc(way, $box.find('.'+opts.contentClass).stop(),
			        		function(){ // out func
				        		
				        		$(this).remove();

				        		// chopper le contenu à afficher
				        		$this = $siblings.eq(num);

				        		// refaire clone or not
				        		$this = cloneOrNot($this, true, $prev);
				        		// refaire adjust content size or not
				        		adjustOrNot($this);

				        		// next on click content ?
				        		nextSiblingOnClick($this);

				        		// apparition du nouveau contenu
				        		// $this.hide().stop()
				        		// .fadeIn(opts.siblingInSpeed); // in func
				        		opts.siblingInFunc(way, $this.hide().stop());

				        		index = num;
				        		refreshNav(); // masquer prev/nav quand on arrive au bout
			        		});
			        	}
		        	}
		        	function refreshNav(){ // maj les btns prev/next
		        		if(index >= $siblings.length-1){
		        			$prev.css({opacity:1, cursor:'pointer'});
		        			$next.css({opacity:0, cursor:'auto'});
		        		} else if(index<=0){
		        			$prev.css({opacity:0, cursor:'auto'});
		        			$next.css({opacity:1, cursor:'pointer'});
		        		} else {
		        			$prev.css({opacity:1, cursor:'pointer'});
		        			$next.css({opacity:1, cursor:'pointer'});
		        		}
		        	}
		        	refreshNav();
		        	nextSiblingOnClick($tmp);

		        	function nextSiblingOnClick($this){
		        		if(opts.nextSiblingOnClick){
		        			$this.on('click.lightbox_sibling', function(e){
				        		e.preventDefault();
				        		e.stopPropagation();
				        		goBy(1);
		        			});
		        		}
		        	}

		        	/*
		        	mmmh
		        	- css de la nav ?
		        	- quelle transition entre 2 slides ?

		        	 */
		        	
		        } // end of siblings stuff -----------------------------------

	        	// ouverture de la lightbox
	        	$box.fadeIn(opts.contentSpeed);
	        }
	    });

		
		return this;
	};
})(jQuery);